import { GAME_SPRITE_SIDE } from '../loader/assets.mjs';

/**
 * @typedef { import("../types").Entity } Entity
 */

/**
 * Creates a new enemy at the given position
 * @param {number} x The starting and home X coordinate of the enemy
 * @param {number} y The starting and home Y coordinate of the enemy
 * @param {string} sprite_closed The sprite to use for the closed door
 * @param {string} sprite_opened The sprite to use for the opened door
 * @param {boolean} is_default_opened Whether the door should be created in an opened state
 * @returns {Entity}
 */
export function createDoor(x, y, sprite_closed, sprite_opened, is_default_opened) {
  return {
    controller_door_component: {
      is_opened: is_default_opened,
      sprite_closed: sprite_closed,
      sprite_opened: sprite_opened,
    },
    position_component: {
      x: x,
      y: y,
    },
    collision_component: {
      is_enabled: !is_default_opened,
      x_offset: -GAME_SPRITE_SIDE / 2,
      y_offset: -GAME_SPRITE_SIDE / 2,
      width: GAME_SPRITE_SIDE,
      height: GAME_SPRITE_SIDE,
    },
    render_simple_sprite_component: {
      is_displayed: true,
      sprite: is_default_opened ? sprite_opened : sprite_closed,
      width: GAME_SPRITE_SIDE,
      height: GAME_SPRITE_SIDE,
    },
  };
}
