import { GAME_SPRITE_SIDE } from '../loader/assets.mjs';

/**
 * @typedef { import("../types").Entity } Entity
 */

/**
 * Creates a new enemy at the given position
 * @param {string} sprite The set of sprites to use for this enemy
 * @param {number} x The starting and home X coordinate of the enemy
 * @param {number} y The starting and home Y coordinate of the enemy
 * @param {number} jx The jail X coordinate of the enemy
 * @param {number} jy The jail Y coordinate of the enemy
 * @returns {Entity}
 */
export function createEnemyBasic(sprite, x, y, jx, jy) {
  return {
    behavior_ai_basic_component: {
      state: 'idle',
      cycle: 0,
      search_cooldown: 0,
      attack_cooldown: 0,
      home: { x: x, y: y },
      target: { x: x, y: y },
      jail: { x: jx, y: jy },
      speed: GAME_SPRITE_SIDE * 0.25,
      dragging_speed: 2,
      enemy_range: GAME_SPRITE_SIDE * 0.75,
      home_range: GAME_SPRITE_SIDE * 1.25,
      effects_to_add: [
        { effect: 'downed', text: 'Thrown to the ground!' },
        { effect: 'tied_hands', text: 'Arms tied!' },
        { effect: 'tied_legs', text: 'Legs tied!' },
        { effect: 'blindfolded', text: 'Blindfold added!' },
        { effect: 'gagged', text: 'Gagged!' },
        { effect: 'leashed', text: 'Leash added!' },
      ],
    },
    collision_component: {
      is_enabled: true,
      x_offset: -GAME_SPRITE_SIDE / 2,
      y_offset: -GAME_SPRITE_SIDE / 2,
      width: GAME_SPRITE_SIDE,
      height: GAME_SPRITE_SIDE,
    },
    controller_movement_component: {
      enabled: true,
      has_collision: false,
      delta_x: 0,
      delta_y: 0,
      stamina_consumption: 0,
    },
    angle_component: {
      angle: Math.PI,
    },
    position_component: {
      x: x,
      y: y,
    },
    vision_component: {
      front: GAME_SPRITE_SIDE * 4,
      sides: GAME_SPRITE_SIDE * 2,
      back: GAME_SPRITE_SIDE * 1.5,
    },
    sprite_controller_enemy_basic_component: {
      is_displayed: true,
      sprite_prefix: 'enemy_sprite_' + sprite,
      fallback_sprite: 'enemy_sprite_' + sprite + '_d',
      direction: 'down',
    },
    render_simple_sprite_component: {
      is_displayed: true,
      sprite: 'enemy_sprite_' + sprite + '_d',
      width: GAME_SPRITE_SIDE,
      height: GAME_SPRITE_SIDE,
    },
  };
}
