import { GAME_SPRITE_SIDE } from '../loader/assets.mjs';

/**
 * @typedef { import("../types").Entity } Entity
 */

/**
 * Creates a new mimic enemy at the given position
 * @param {string} sprite The set of sprites to use for this enemy
 * @param {number} x The starting and home X coordinate of the enemy
 * @param {number} y The starting and home Y coordinate of the enemy
 * @returns {Entity}
 */
export function createEnemyMimic(sprite, x, y) {
  return {
    behavior_ai_mimic_component: {
      state: 'idle',
      last_target_position: {
        x: 0,
        y: 0,
      },
      animation_cycle: 0,
      digest_cycle: 0,
      digest_time: 80,
      digest_rumble: 10,
      active_range: GAME_SPRITE_SIDE * 2,
      grab_range: GAME_SPRITE_SIDE * 1.25,
      effects: [
        { timer: 70, type: 'gagged', text: 'Gagged!' },
        { timer: 50, type: 'blindfolded', text: 'Blindfold added!' },
        { timer: 30, type: 'tied_hands', text: 'Arms tied!' },
        { timer: 22, type: 'teased', text: 'Vibrating egg added!' },
        { timer: 20, type: 'tied_legs', text: 'Legs tied!' },
        { timer: 10, type: 'leashed', text: 'Leash added!' },
        { timer: 1, type: 'downed', text: 'Thrown out!' },
      ],
    },
    position_component: {
      x: x,
      y: y,
    },
    render_simple_sprite_component: {
      is_displayed: true,
      sprite: 'enemy_sprite_' + sprite + '_idle',
      width: GAME_SPRITE_SIDE,
      height: GAME_SPRITE_SIDE,
    },
    sprite_controller_mimic_component: {
      is_displayed: true,
      sprite_prefix: 'enemy_sprite_' + sprite,
      status: 'idle',
      fallback_sprite: 'enemy_sprite_' + sprite + '_idle',
    },
  };
}
