import { GAME_SPRITE_SIDE } from '../loader/assets.mjs';

/**
 * @typedef { import("../types").Entity } Entity
 */

/**
 * Creates a new enemy at the given position
 * @param { string } sprite The set of sprites to use for this enemy
 * @param { number } x The starting and home X coordinate of the enemy
 * @param { number } y The starting and home Y coordinate of the enemy
 * @param { Array<{ x: number, y: number }> } patrol_points The patrol points for the enemy to use
 * @returns { Entity }
 */
export function createEnemyPatrol(sprite, x, y, patrol_points) {
  return {
    behavior_ai_component: {
      behavior: {
        next_state_selector: [
          { status: { is_in_state: 'patrol', has_reached_patrol_point: true }, state: 'scan' },
          { status: { is_in_state: 'scan', has_finished_scan_cycle: true }, state: 'patrol' },
        ],
      },
      status: {
        current_state: 'patrol',
        current_patrol_index: 0,
        increment_counter: 0,
        tick_counter: 0,
      },
      configuration: {
        scan_in_place: {
          increments_before_event: 10,
          rotation_percent_per_increment: 10,
          ticks_before_increment: 2,
        },
        patrol: {
          speed: GAME_SPRITE_SIDE * 0.15,
          waypoints: patrol_points,
          range_until_point_reached: GAME_SPRITE_SIDE * 0.25,
        },
        capture: {
          dragging_speed: GAME_SPRITE_SIDE * 0.2,
        },
      },
    },
    controller_movement_component: {
      enabled: true,
      has_collision: false,
      delta_x: 0,
      delta_y: 0,
      stamina_consumption: 0,
    },
    position_component: {
      x: x,
      y: y,
    },
    angle_component: {
      angle: Math.PI,
    },
    sprite_controller_enemy_basic_component: {
      is_displayed: true,
      direction: 'down',
      sprite_prefix: 'enemy_sprite_' + sprite,
      fallback_sprite: 'enemy_sprite_' + sprite + '_d',
    },
    render_simple_sprite_component: {
      is_displayed: true,
      sprite: 'enemy_sprite_' + sprite + '_d',
      width: GAME_SPRITE_SIDE,
      height: GAME_SPRITE_SIDE,
    },
  };
}
