import { GAME_SPRITE_SIDE } from '../loader/assets.mjs';

/**
 * @typedef { import("../types").Entity } Entity
 */

/**
 * Create a new in-world "add effect" item
 * @param {number} x the x coordinate to create the effect item at
 * @param {number} y the y coordinate to create the effect item at
 * @param {string} sprite the sprite to use
 * @param {string[]} effect_type the effects to create when touching the effect item
 * @param {number} duration_s the duration of the effects when touching the effect item, in seconds
 * @param {boolean} is_infinite Whether the created effects are infinite
 * @returns {Entity}
 */
export function createItemThatAddsEffects(x, y, sprite, effect_type, duration_s, is_infinite) {
  return {
    position_component: {
      x: x,
      y: y,
    },
    render_simple_sprite_component: {
      is_displayed: true,
      sprite: sprite,
      width: GAME_SPRITE_SIDE,
      height: GAME_SPRITE_SIDE,
    },
    on_contact_action_component: {
      range: 5,
      delete_on_touch: true,
      action: {
        type: 'effects',
        effects: effect_type.map((e) => ({ type: e, duration_s: duration_s, is_infinite: is_infinite })),
      },
    },
  };
}
