import { GAME_SPRITE_SIDE } from '../loader/assets.mjs';

/**
 * @typedef { import("../types").Entity } Entity
 */

/**
 * Creates a new player at the given position
 * @param {number} x The starting X coordinate of the player
 * @param {number} y The starting Y coordinate of the player
 * @returns {Entity}
 */
export function createPlayer(x, y) {
  return {
    controller_camera_component: {
      camera_follows_entity: true,
    },
    behavior_keyboard_component: {
      enabled: false,
      direction: 'down',
      run_stamina_cost: 1,
      slowed_speed: 0.01 * GAME_SPRITE_SIDE,
      normal_speed: 0.1 * GAME_SPRITE_SIDE,
      running_speed: 0.4 * GAME_SPRITE_SIDE,
      struggle_mode: 'inactive',
      struggle_counter: 0,
      previous_struggle_keys: [],
      struggle_stamina_cost: 2,
    },
    controller_movement_component: {
      enabled: true,
      has_collision: true,
      delta_x: 0,
      delta_y: 0,
      stamina_consumption: 0,
    },
    controller_stamina_regen_component: {
      stamina_regen_cooldown: 0,
      max_stamina_regen_cooldown: 20,
      stamina_regen_quantity_per_tick: 0.33,
    },
    basic_enemy_ai_detection_component: {
      can_be_detected: true,
      can_be_jailed: true,
    },
    contact_component: {
      hitbox_area: 0.4 * GAME_SPRITE_SIDE,
    },
    contained_component: {
      is_inside_container: false,
    },
    effects_component: {
      effects: [],
    },
    faceset_component: {
      default: 'face_standard',
      image: '',
    },
    inventory_component: {
      inventory: [
        {
          type: 'gold',
          quantity: 0,
        },
      ],
    },
    mimic_ai_detection_component: {
      can_be_detected: true,
      can_be_captured: true,
    },
    position_component: {
      x: x,
      y: y,
    },
    stamina_component: {
      stamina: 100,
      max_stamina: 100,
    },
    render_blindfold_component: {
      blindfold_enabled: false,
      visible_tiles_around: 2,
    },
    render_hud_component: {
      enable_effects: true,
      enable_faceset: true,
      enable_inventory: true,
      enable_status: true,
      enable_struggle_bar: true,
    },
    render_struggle_bar_component: {
      enabled: false,
      activated: false,
      progress_percent: 0,
      keys: [],
    },
    render_simple_sprite_component: {
      is_displayed: true,
      sprite: 'player_sprite_default_down',
      width: GAME_SPRITE_SIDE,
      height: GAME_SPRITE_SIDE,
    },
    sprite_controller_player_component: {
      is_displayed: true,
      direction: 'down',
      direction_override: undefined,
      type_modifier: undefined,
      sprite_prefix: 'player_sprite',
      fallback_sprite: 'player_sprite_default_down',
    },
  };
}
