/**
 * @typedef { import("../types").ImageAssetPartialInformation } ImageAssetPartialInformation
 */

const GAME_SPRITE_SIDE = 40;

const spritesheet_game_player = [
  { type: 'default', x: 0, y: 0 },
  { type: 'gag', x: 0, y: 40 },
  { type: 'blind', x: 0, y: 80 },
  { type: 'tie', x: 0, y: 120 },
  { type: 'gag_blind', x: 0, y: 160 },
  { type: 'tie_gag', x: 0, y: 200 },
  { type: 'tie_blind', x: 0, y: 240 },
  { type: 'tie_gag_blind', x: 0, y: 280 },
  { type: 'suspension', x: 0, y: 320 },
];

const spritesheet_game_enemy_basic = [
  { type: 'robot', x: 0, y: 360 },
  { type: 'dryad', x: 0, y: 400 },
];

const spritesheet_game_enemy_mimic = [{ type: 'circled_chest', x: 0, y: 440 }];

const spritesheet_game_items = [
  { type: 'potion_red', x: 0, y: 480 },
  { type: 'potion_blue', x: 40, y: 480 },
  { type: 'rope', x: 80, y: 480 },
  { type: 'ballgag', x: 120, y: 480 },
  { type: 'blindfold', x: 160, y: 480 },
  { type: 'vibrator', x: 200, y: 480 },
  { type: 'gold_coins', x: 240, y: 480 },
];

const spritesheet_game_map = [
  { type: 'floor_bricks', x: 320, y: 0 },
  { type: 'floor_sparse', x: 360, y: 0 },
  { type: 'floor_smooth', x: 400, y: 0 },
  { type: 'wall_deco01', x: 320, y: 40 },
  { type: 'wall_deco02', x: 360, y: 40 },
  { type: 'wall_smooth', x: 400, y: 40 },
  { type: 'grid_wall_v', x: 320, y: 80 },
  { type: 'grid_wall_h', x: 360, y: 80 },
  { type: 'grid_wall_cl', x: 400, y: 80 },
  { type: 'grid_wall_cr', x: 440, y: 80 },
  { type: 'grid_door_hc', x: 320, y: 120 },
  { type: 'grid_door_ho', x: 360, y: 120 },
  { type: 'grid_door_vc', x: 400, y: 120 },
  { type: 'grid_door_vo', x: 440, y: 120 },
];

const spritesheet_game_ui = [
  { type: 'key_up', x: 0, y: 560 },
  { type: 'key_left', x: 40, y: 560 },
  { type: 'key_down', x: 80, y: 560 },
  { type: 'key_right', x: 120, y: 560 },
  { type: 'key_space', x: 160, y: 560 },
];

const spritesheet_faces = [
  { asset: 'face_standard', x: 0, y: 0 },
  { asset: 'face_suspension', x: 900, y: 0 },
  { asset: 'face_suspension_s', x: 1350, y: 0 },
  { asset: 'face_surprised', x: 0, y: 600 },
  { asset: 'face_gag', x: 450, y: 600 },
  { asset: 'face_blind', x: 900, y: 600 },
  { asset: 'face_gag_blind', x: 1350, y: 600 },
  { asset: 'face_tie', x: 0, y: 1200 },
  { asset: 'face_tie_s', x: 0, y: 1800 },
  { asset: 'face_gag_tie', x: 450, y: 1200 },
  { asset: 'face_gag_tie_s', x: 450, y: 1800 },
  { asset: 'face_blind_tie', x: 900, y: 1200 },
  { asset: 'face_blind_tie_s', x: 900, y: 1800 },
  { asset: 'face_gag_blind_tie', x: 1350, y: 1200 },
  { asset: 'face_gag_blind_tie_s', x: 1350, y: 1800 },
];

const spritesheet_status = [
  { asset: 'effect_gagged', x: 0, y: 0 },
  { asset: 'effect_silenced', x: 300, y: 0 },
  { asset: 'effect_no_consumables', x: 600, y: 0 },
  { asset: 'effect_tied_hands', x: 0, y: 300 },
  { asset: 'effect_tied_legs', x: 300, y: 300 },
  { asset: 'effect_leashed', x: 600, y: 300 },
  { asset: 'effect_blinded', x: 0, y: 600 },
  { asset: 'effect_blindfolded', x: 300, y: 600 },
  { asset: 'effect_hanging', x: 600, y: 600 },
  { asset: 'effect_teased', x: 0, y: 900 },
  { asset: 'effect_distracted', x: 300, y: 900 },
  { asset: 'effect_downed', x: 600, y: 900 },
  { asset: 'effect_suspended', x: 0, y: 1200 },
];

/** @type {{ [file_url: string]: { [key_name: string]: ImageAssetPartialInformation } }} */
const imageDescriptions = {
  'assets/images/game_sprites.png': {},
  'assets/images/faces_sprites.png': {},
  'assets/images/status_sprites.png': {},
};

for (const sprite of spritesheet_game_player) {
  const group_prefix = 'player_sprite_' + sprite.type;
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_d'] = {
    origin: { x: sprite.x + 0, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_r'] = {
    origin: { x: sprite.x + 40, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_u'] = {
    origin: { x: sprite.x + 80, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_l'] = {
    origin: { x: sprite.x + 120, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_down'] = {
    origin: { x: sprite.x + 160, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
}

for (const sprite of spritesheet_game_enemy_basic) {
  const group_prefix = 'enemy_sprite_' + sprite.type;
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_d'] = {
    origin: { x: sprite.x + 0, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_r'] = {
    origin: { x: sprite.x + 40, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_u'] = {
    origin: { x: sprite.x + 80, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_l'] = {
    origin: { x: sprite.x + 120, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
}

for (const sprite of spritesheet_game_enemy_mimic) {
  const group_prefix = 'enemy_sprite_' + sprite.type;
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_idle'] = {
    origin: { x: sprite.x + 0, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_attack1'] = {
    origin: { x: sprite.x + 40, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_attack2'] = {
    origin: { x: sprite.x + 80, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_digest1'] = {
    origin: { x: sprite.x + 120, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
  imageDescriptions['assets/images/game_sprites.png'][group_prefix + '_digest2'] = {
    origin: { x: sprite.x + 160, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
}

for (const sprite of spritesheet_game_items) {
  const sprite_name = 'item_' + sprite.type;
  imageDescriptions['assets/images/game_sprites.png'][sprite_name] = {
    origin: { x: sprite.x, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
}

for (const sprite of spritesheet_game_map) {
  const sprite_name = 'map_' + sprite.type;
  imageDescriptions['assets/images/game_sprites.png'][sprite_name] = {
    origin: { x: sprite.x, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
}

for (const sprite of spritesheet_game_ui) {
  const sprite_name = 'ui_' + sprite.type;
  imageDescriptions['assets/images/game_sprites.png'][sprite_name] = {
    origin: { x: sprite.x, y: sprite.y, w: 40, h: 40 },
    target: { w: GAME_SPRITE_SIDE, h: GAME_SPRITE_SIDE },
  };
}

for (const face of spritesheet_faces) {
  imageDescriptions['assets/images/faces_sprites.png'][face.asset] = {
    origin: { x: face.x, y: face.y, w: 450, h: 600 },
    target: { w: 110, h: 150 },
  };
}

for (const face of spritesheet_status) {
  imageDescriptions['assets/images/status_sprites.png'][face.asset] = {
    origin: { x: face.x, y: face.y, w: 300, h: 300 },
    target: { w: 40, h: 40 },
  };
}

export { imageDescriptions, GAME_SPRITE_SIDE };
