import { getDistance } from '../utils/distance.mjs';

/**
 * @typedef { import("../engine.mjs").Game } Game
 */

export function getControllerContactSystem() {
  /**
   * @param {Game} game
   * @param {number} elapsedTicks
   */
  return function controllerContactSystem(game, elapsedTicks) {
    for (const main_entity of game.entities) {
      if (!main_entity.contact_component) continue;
      if (!main_entity.position_component) continue;
      for (const touched_entity of game.entities) {
        if (!touched_entity.on_contact_action_component) continue;
        if (!touched_entity.position_component) continue;

        const distance = getDistance(main_entity.position_component, touched_entity.position_component);
        if (distance >= main_entity.contact_component.hitbox_area + touched_entity.on_contact_action_component.range)
          continue;

        if (touched_entity.on_contact_action_component.action.type === 'gold') {
          if (!main_entity.inventory_component) continue;
          const gold = main_entity.inventory_component.inventory.find((i) => i.type === 'gold');
          if (!gold || gold.type !== 'gold') return;
          gold.quantity += touched_entity.on_contact_action_component.action.quantity;
          game.createTextParticle(
            'Gold! (x' + touched_entity.on_contact_action_component.action.quantity + ')',
            touched_entity.position_component.x,
            touched_entity.position_component.y
          );
        }

        if (touched_entity.on_contact_action_component.action.type === 'stamina') {
          if (!main_entity.stamina_component) continue;
          main_entity.stamina_component.stamina += touched_entity.on_contact_action_component.action.quantity;
          if (main_entity.stamina_component.stamina > main_entity.stamina_component.max_stamina) {
            main_entity.stamina_component.stamina = main_entity.stamina_component.max_stamina;
          }
          game.createTextParticle(
            'Samina restored! (+' + touched_entity.on_contact_action_component.action.quantity + ')',
            touched_entity.position_component.x,
            touched_entity.position_component.y
          );
        }

        if (touched_entity.on_contact_action_component.action.type === 'effects') {
          if (!main_entity.effects_component) continue;
          let offset = 0;
          for (const effect of touched_entity.on_contact_action_component.action.effects) {
            main_entity.effects_component.effects.push({
              type: effect.type,
              isInfinite: effect.is_infinite,
              remainingFrameCount: (effect.duration_s || 0) * 20,
            });
            game.createTextParticle(
              `Effect: ${effect.type}!`,
              touched_entity.position_component.x,
              touched_entity.position_component.y + offset
            );
            offset -= 10;
          }
        }

        if (touched_entity.on_contact_action_component.delete_on_touch) {
          game.entities = game.entities.filter((i) => i != touched_entity);
        }
      }
    }
  };
}
