import { getFacesetModifier, hasBlindingEffect } from '../effects.mjs';

/**
 * @typedef { import("../engine.mjs").Game } Game
 */

export function getControllerEffectsSystem() {
  /**
   * @param {Game} game
   * @param {number} elapsedTicks
   */
  return function controllerEffectsSystem(game, elapsedTicks) {
    for (const entity of game.entities) {
      if (!entity.effects_component) continue;
      if (!entity.position_component) continue;

      if (entity.effects_component.effects.some((e) => e.type === 'teased')) {
        if (!entity.effects_component.effects.some((e) => e.type === 'distracted') && Math.random() < 0.01) {
          game.createTextParticle(
            'Your vibrator teases you!',
            entity.position_component.x,
            entity.position_component.y
          );
          entity.effects_component.effects.push({
            type: 'distracted',
            remainingFrameCount: 20 * 10,
          });
        }
      }

      if (entity.effects_component.effects.some((e) => e.type === 'distracted')) {
        if (!entity.effects_component.effects.some((e) => e.type === 'downed') && Math.random() < 0.002) {
          game.createTextParticle(
            "You're distracted and stumble to the ground!",
            entity.position_component.x,
            entity.position_component.y
          );
          entity.effects_component.effects.push({
            type: 'downed',
            isInfinite: true,
          });
        }
      }

      for (const effect of entity.effects_component.effects) {
        if (!effect.isInfinite && effect.remainingFrameCount) {
          effect.remainingFrameCount -= elapsedTicks;
        }
      }
      entity.effects_component.effects = entity.effects_component.effects.filter(
        (e) => e.isInfinite || e.remainingFrameCount > 0
      );

      // Update the HUD component
      if (entity.faceset_component && entity.behavior_keyboard_component) {
        const image_path = getFacesetModifier(
          entity.effects_component.effects,
          entity.behavior_keyboard_component.struggle_counter
        );
        entity.faceset_component.image = image_path;
      }

      if (entity.render_blindfold_component) {
        entity.render_blindfold_component.blindfold_enabled = hasBlindingEffect(entity.effects_component.effects);
        entity.render_blindfold_component.visible_tiles_around = 2;
      }
    }
  };
}
