import { collidesWithEntity, collidesWithMap } from '../utils/collision.mjs';

/**
 * @typedef { import("../engine.mjs").Game } Game
 */

export function getControllerMovementSystem() {
  /**
   * @param {Game} game
   * @param {number} elapsedTicks
   */
  return function controllerMovementSystem(game, elapsedTicks) {
    for (const entity of game.entities) {
      if (!entity.controller_movement_component) continue;
      if (!entity.position_component) continue;

      if (!entity.controller_movement_component.enabled) continue;

      if (entity.controller_movement_component.delta_x === 0 && entity.controller_movement_component.delta_y === 0)
        continue;

      // TODO: Turn the collision into an independent component instead of hardcoding that data in a flag
      if (entity.controller_movement_component.has_collision) {
        const collision_data = {
          x: entity.position_component.x - 15 + entity.controller_movement_component.delta_x,
          y: entity.position_component.y - 15 + entity.controller_movement_component.delta_y,
          width: 30,
          height: 30,
        };
        if (collidesWithMap(game, collision_data, false) || collidesWithEntity(game, collision_data)) {
          continue;
        }
      }

      entity.position_component.x += entity.controller_movement_component.delta_x;
      entity.position_component.y += entity.controller_movement_component.delta_y;

      if (entity.controller_movement_component.stamina_consumption > 0) {
        if (entity.controller_stamina_regen_component)
          entity.controller_stamina_regen_component.stamina_regen_cooldown =
            entity.controller_stamina_regen_component.max_stamina_regen_cooldown;
        if (entity.stamina_component)
          entity.stamina_component.stamina -= entity.controller_movement_component.stamina_consumption;

        // TODO: Turn this into an independent particle emitter component
        const particleX =
          entity.controller_movement_component.delta_x > 0
            ? -15
            : entity.controller_movement_component.delta_x < 0
            ? 15
            : 0;
        const particleY =
          entity.controller_movement_component.delta_y > 0
            ? -15
            : entity.controller_movement_component.delta_y < 0
            ? 15
            : 0;
        game.createBlinkParticle(entity.position_component.x + particleX, entity.position_component.y + particleY);
      }
    }
  };
}
