/**
 * @typedef { import("../engine.mjs").Game } Game
 */

/**
 * @param {{ [key: string]: ImageBitmap }} images The loaded images to use by the renderer
 */
export function getHudInventorySystem(images) {
  /**
   * @param {Game} game The current game
   */
  return function hudInventorySystem(game) {
    for (const entity of game.entities) {
      if (!entity.render_hud_component) continue;
      if (!entity.render_hud_component.enable_inventory) continue;
      if (!entity.inventory_component) continue;
      game.context.textAlign = 'start';
      game.context.fillStyle = 'white';
      game.context.strokeStyle = 'transparent';
      game.context.font = '12px sans-serif';
      let startXPosition = 160;
      let startYPosition = 570;
      let currentXPosition = startXPosition;
      for (const item of entity.inventory_component.inventory) {
        if (item.type === 'gold') {
          game.context.drawImage(images['item_gold_coins'], currentXPosition - 10, startYPosition - 10, 40, 40);
          currentXPosition += 30;
          const metrics = game.context.measureText(`x${item.quantity}`);
          game.context.fillText(`x${item.quantity}`, currentXPosition, startYPosition + 20);
          currentXPosition += 20 + metrics.width;
        }
      }
    }
  };
}
