import { GAME_SPRITE_SIDE } from '../loader/assets.mjs';

/**
 * @typedef { import("../engine.mjs").Game } Game
 */

/**
 * @param {{ [key: string]: ImageBitmap }} images The loaded images to use by the renderer
 */
export function getHudStruggleBarSystem(images) {
  /**
   * @param {Game} game
   */
  return function hudStruggleBarSystem(game) {
    for (const entity of game.entities) {
      if (!entity.render_hud_component) continue;
      if (!entity.render_hud_component.enable_struggle_bar) continue;
      if (!entity.position_component) continue;
      if (!entity.render_struggle_bar_component) continue;
      if (!entity.render_struggle_bar_component.enabled) continue;

      if (!entity.render_struggle_bar_component.activated) {
        game.renderer.renderImage(
          images['ui_key_space'],
          entity.position_component.x - GAME_SPRITE_SIDE / 2,
          entity.position_component.y - GAME_SPRITE_SIDE / 2 + GAME_SPRITE_SIDE,
          GAME_SPRITE_SIDE,
          GAME_SPRITE_SIDE
        );
        continue;
      }

      // Draw the key to press
      if (!entity.render_struggle_bar_component.keys.includes('up')) {
        game.renderer.renderImage(
          images['ui_key_up'],
          entity.position_component.x - GAME_SPRITE_SIDE / 2,
          entity.position_component.y - GAME_SPRITE_SIDE / 2 - GAME_SPRITE_SIDE,
          GAME_SPRITE_SIDE,
          GAME_SPRITE_SIDE
        );
      }
      if (!entity.render_struggle_bar_component.keys.includes('left')) {
        game.renderer.renderImage(
          images['ui_key_left'],
          entity.position_component.x - GAME_SPRITE_SIDE / 2 - GAME_SPRITE_SIDE,
          entity.position_component.y - GAME_SPRITE_SIDE / 2,
          GAME_SPRITE_SIDE,
          GAME_SPRITE_SIDE
        );
      }
      if (!entity.render_struggle_bar_component.keys.includes('down')) {
        game.renderer.renderImage(
          images['ui_key_down'],
          entity.position_component.x - GAME_SPRITE_SIDE / 2,
          entity.position_component.y - GAME_SPRITE_SIDE / 2 + GAME_SPRITE_SIDE,
          GAME_SPRITE_SIDE,
          GAME_SPRITE_SIDE
        );
      }
      if (!entity.render_struggle_bar_component.keys.includes('right')) {
        game.renderer.renderImage(
          images['ui_key_right'],
          entity.position_component.x - GAME_SPRITE_SIDE / 2 + GAME_SPRITE_SIDE,
          entity.position_component.y - GAME_SPRITE_SIDE / 2,
          GAME_SPRITE_SIDE,
          GAME_SPRITE_SIDE
        );
      }
      // Draw the struggle bar
      game.renderer.renderWorldRect(entity.position_component.x - 12, entity.position_component.y - 20, 24, 5, 'grey');
      game.renderer.renderWorldRect(entity.position_component.x - 10, entity.position_component.y - 20, 20, 5, 'white');
      const bar_progress_px = 20 * entity.render_struggle_bar_component.progress_percent;
      game.renderer.renderWorldRect(
        entity.position_component.x - 10,
        entity.position_component.y - 20,
        bar_progress_px,
        5,
        'purple'
      );
    }
  };
}
