/**
 * @typedef { import("../engine.mjs").Game } Game
 */

export function getHudTextEventsSystem() {
  /**
   * @param {Game} game
   */
  return function hudTextEventsSystem(game) {
    let eventY = 600 - 30;
    for (const event of game.events) {
      if (event.type !== 'text') continue;
      event.timeout--;
      if (event.timeout <= 0) continue;
      drawTextEvent(game, event, eventY);
      eventY -= 30;
    }

    game.events = game.events.filter((e) => e.timeout > 0);
  };

  /**
   * @param {Game} game
   * @param {{ type: "text", textContent: string, timeout: number }} event
   * @param {number} eventY
   */
  function drawTextEvent(game, event, eventY) {
    game.context.textAlign = 'left';
    game.context.fillStyle = 'rgba(255, 255, 255, ' + Math.min(event.timeout / 300, 1.0) + ')';
    game.context.strokeStyle = 'transparent';
    game.context.font = '12px sans-serif';
    const measure = game.context.measureText(event.textContent);
    const width = measure.actualBoundingBoxRight - measure.actualBoundingBoxLeft;
    game.context.strokeStyle = 'rgba(127, 127, 127, ' + Math.min(event.timeout / 300, 1.0) + ')';
    game.context.fillRect(800 - width - 30, eventY, width + 20, 20);
    game.context.strokeRect(800 - width - 30, eventY, width + 20, 20);
    game.context.fillStyle = 'rgba(0, 0, 0, ' + Math.min(event.timeout / 300, 1.0) + ')';
    game.context.strokeStyle = 'transparent';
    game.context.fillText(event.textContent, 800 - width - 20, eventY + 15, width);
  }
}
