/**
 * @typedef { import("../engine.mjs").Game } Game
 */

export function getRenderMapSystem() {
  /**
   * @param {Game} game
   */
  return function renderMapSystem(game) {
    const tile_width = game.mapData.tiles.tile_width;
    const tile_height = game.mapData.tiles.tile_height;
    const x_start = Math.floor(game.mapData.offsetX / tile_width);
    const y_start = Math.floor(game.mapData.offsetY / tile_height);
    const x_remainder =
      game.mapData.offsetX >= tile_width ? (tile_width - game.mapData.offsetX) % tile_width : -game.mapData.offsetX;
    const y_remainder =
      game.mapData.offsetY >= tile_height ? (tile_height - game.mapData.offsetY) % tile_height : -game.mapData.offsetY;
    const x_end = Math.ceil((game.mapData.offsetX + game.mapData.render_width) / tile_width);
    const y_end = Math.ceil((game.mapData.offsetY + game.mapData.render_height) / tile_height);
    for (let y = y_start; y < y_end; ++y) {
      const content_row = game.mapData.tiles.content[y];
      for (let x = x_start; x < x_end; ++x) {
        const content_value = content_row ? content_row[x] || 0 : 0;
        let method = game.mapData.tiles.map[content_value];
        method(game, (x - x_start) * tile_width + x_remainder, (y - y_start) * tile_height + y_remainder);
      }
    }
  };
}
