/**
 * @typedef { import("../engine.mjs").Game } Game
 */

export function getRenderParticlesSystem() {
  /**
   * @param {Game} game
   */
  return function renderParticlesSystem(game) {
    for (const particle of game.particles) {
      particle.x += particle.dirX;
      particle.y += particle.dirY;
      particle.dirX += particle.decayX;
      particle.dirY += particle.decayY;
      particle.currentTimer++;
      if (particle.totalTimer < particle.currentTimer) continue;
      if (particle.type === 'text') {
        const alpha = 1 - particle.currentTimer / particle.totalTimer;
        game.context.fillStyle = 'rgba(255,255,255,' + alpha + ')';
        game.context.strokeStyle = 'transparent';
        game.context.font = '8px sans-serif';
        game.context.textAlign = 'center';
        game.context.fillText(particle.text, particle.x - game.mapData.offsetX, particle.y - game.mapData.offsetY);
      }
      if (particle.type === 'blink') {
        game.renderer.renderWorldRect(particle.x - 1, particle.y - 1, 2, 2, particle.color);
      }
    }

    game.particles = game.particles.filter((p) => p.totalTimer >= p.currentTimer);
  };
}
