/**
 * @typedef { import("../engine.mjs").Game } Game
 */

/**
 * @param {{ [key: string]: ImageBitmap }} images The loaded images to use by the renderer
 */
export function getRenderSimpleSpriteSystem(images) {
  /**
   * @param {Game} game
   */
  return function renderSimpleSpriteSystem(game) {
    for (const entity of game.entities) {
      if (!entity.render_simple_sprite_component) continue;
      if (!entity.position_component) continue;
      if (!entity.render_simple_sprite_component.is_displayed) continue;

      if (!images[entity.render_simple_sprite_component.sprite]) {
        console.warn(`Could not find image: ${entity.render_simple_sprite_component.sprite}`);
      }
      game.renderer.renderImage(
        images[entity.render_simple_sprite_component.sprite],
        entity.position_component.x - entity.render_simple_sprite_component.width / 2,
        entity.position_component.y - entity.render_simple_sprite_component.height / 2,
        entity.render_simple_sprite_component.width,
        entity.render_simple_sprite_component.height
      );
    }
  };
}
