import { getVisionBox } from '../utils/vision-box.mjs';

/**
 * @typedef { import("../engine.mjs").Game } Game
 */

export function getRenderVisionSystem() {
  /**
   * @param {Game} game
   */
  return function renderVisionSystem(game) {
    for (const entity of game.entities) {
      if (!entity.angle_component) continue;
      if (!entity.vision_component) continue;
      if (!entity.position_component) continue;
      const vision_box = getVisionBox(
        game,
        entity.position_component,
        entity.angle_component.angle,
        entity.vision_component.front,
        entity.vision_component.sides,
        entity.vision_component.back
      );
      game.renderer.renderWorldRect(
        vision_box.x,
        vision_box.y,
        vision_box.width,
        vision_box.height,
        'rgba(255, 255, 255, 0.2)'
      );
    }
  };
}
