/**
 * @typedef { import("../engine.mjs").Game } Game
 */

export function getSpriteControllerPlayerSystem() {
  /**
   * @param {Game} game
   * @param {number} elapsedTicks
   */
  return function spriteControllerPlayerSystem(game, elapsedTicks) {
    for (const entity of game.entities) {
      if (!entity.sprite_controller_player_component) continue;
      if (!entity.behavior_keyboard_component) continue;
      if (!entity.render_simple_sprite_component) continue;
      if (!entity.contained_component) continue;
      if (!entity.effects_component) continue;

      let sprite_prefix = '';
      if (entity.effects_component.effects.some((e) => e.type === 'tied_hands')) sprite_prefix += '_tie';
      if (entity.effects_component.effects.some((e) => e.type === 'gagged')) sprite_prefix += '_gag';
      if (entity.effects_component.effects.some((e) => e.type === 'blindfolded')) sprite_prefix += '_blind';
      if (entity.effects_component.effects.some((e) => e.type === 'suspended')) sprite_prefix = '_suspension';
      entity.sprite_controller_player_component.type_modifier = sprite_prefix.substring(1) || undefined;
      if (entity.effects_component.effects.some((e) => e.type === 'downed')) {
        entity.sprite_controller_player_component.direction_override = 'ground';
      } else {
        entity.sprite_controller_player_component.direction = entity.behavior_keyboard_component.direction;
        entity.sprite_controller_player_component.direction_override = undefined;
      }
      entity.sprite_controller_player_component.is_displayed = !entity.contained_component.is_inside_container;

      entity.render_simple_sprite_component.is_displayed = entity.sprite_controller_player_component.is_displayed;

      // Note: This next part is a bit ugly, and it would be nice to clean it up a bit...
      /** @type {string} */
      let sprite_suffix = '';
      if (entity.sprite_controller_player_component.direction_override) {
        sprite_suffix = { ground: 'down' }[entity.sprite_controller_player_component.direction_override];
      } else {
        sprite_suffix = {
          up: 'u',
          down: 'd',
          left: 'l',
          right: 'r',
        }[entity.sprite_controller_player_component.direction];
      }

      const type_modifier = entity.sprite_controller_player_component.type_modifier
        ? entity.sprite_controller_player_component.type_modifier
        : 'default';

      entity.render_simple_sprite_component.sprite =
        entity.sprite_controller_player_component.sprite_prefix + '_' + type_modifier + '_' + sprite_suffix;
    }
  };
}
