import { getDistance } from '../distance.mjs';
import { getNextPathfindingPoint } from '../pathfinding.mjs';

/**
 * @typedef { import("../../engine.mjs").Game } Game
 * @typedef { import("../../types").Entity } Entity
 */

/**
 * Executes the AI Patrol Behavior for the given entity
 * @param {Game} game
 * @param {Entity} entity
 * @param {number} elapsedTicks
 * @returns { string | undefined }
 */
export function carryAIBehavior(game, entity, elapsedTicks) {
  if (!entity.behavior_ai_component) return;
  if (entity.behavior_ai_component.status.current_state !== 'patrol') return;
  if (!entity.behavior_ai_component.configuration.jail) return;
  if (!entity.controller_movement_component) return;
  if (!entity.position_component) return;
  if (!entity.angle_component) return;

  // Drag the player home
  const target_angle = Math.atan2(
    entity.position_component.y - entity.behavior_ai_component.configuration.jail.y,
    entity.position_component.x - entity.behavior_ai_component.configuration.jail.x
  );
  entity.controller_movement_component.delta_x -=
    entity.behavior_ai_component.configuration.capture.dragging_speed * Math.cos(target_angle);
  entity.controller_movement_component.delta_y -=
    entity.behavior_ai_component.configuration.capture.dragging_speed * Math.sin(target_angle);

  // Force-drag the player away
  // TODO: Rework the way this is done by using a specific "dragging" component that interferes with the player's movement component.
  const target_entity = game.entities.find((e) => e.behavior_keyboard_component);
  if (target_entity && target_entity.position_component) {
    target_entity.position_component.x -=
      entity.behavior_ai_component.configuration.capture.dragging_speed * Math.cos(target_angle);
    target_entity.position_component.y -=
      entity.behavior_ai_component.configuration.capture.dragging_speed * Math.sin(target_angle);
  }

  entity.angle_component.angle = target_angle;
  if (entity.angle_component.angle < 0) {
    entity.angle_component.angle += 2 * Math.PI;
  }
  entity.angle_component.angle %= 2 * Math.PI;
  return;
}
