import { patrolAIBehavior } from './patrol.mjs';
import { scanAIBehavior } from './scan.mjs';

/**
 * @typedef { import("../../engine.mjs").Game } Game
 * @typedef { import("../../types").Entity } Entity
 * @typedef { import("../../types").AIState } AIState
 */

/** @type { { [key in AIState]: (game: Game, entity: Entity, elapsedTicks: number ) => string | undefined }} */
const functions = {
  patrol: patrolAIBehavior,
  scan: scanAIBehavior,
  back_to_home: (_1, _2, _3) => {
    return '';
  },
  capture: (_1, _2, _3) => {
    return '';
  },
  carry: (_1, _2, _3) => {
    return '';
  },
  search: (_1, _2, _3) => {
    return '';
  },
};

/**
 * @param {Game} game
 * @param {Entity} entity
 * @param {number} elapsedTicks
 */
export function complexAIBehavior(game, entity, elapsedTicks) {
  if (!entity.behavior_ai_component) return;
  const method = functions[entity.behavior_ai_component.status.current_state];
  const result = method(game, entity, elapsedTicks);
  const current_status = {
    current_state: entity.behavior_ai_component.status.current_state,
    has_finished_scan_cycle: result === 'scan_cycle_finished',
    has_reached_patrol_point: result === 'patrol_point_reached',
    has_visible_target: false,
    is_at_home: false,
    is_carrying_target: false,
    is_in_capture_range: false,
  };
  const next_state_or_undefined = entity.behavior_ai_component.behavior.next_state_selector.find((state) => {
    if (
      state.status.has_finished_scan_cycle !== undefined &&
      state.status.has_finished_scan_cycle !== current_status.has_finished_scan_cycle
    )
      return false;
    if (
      state.status.has_reached_patrol_point !== undefined &&
      state.status.has_reached_patrol_point !== current_status.has_reached_patrol_point
    )
      return false;
    if (
      state.status.has_visible_target !== undefined &&
      state.status.has_visible_target !== current_status.has_visible_target
    )
      return false;
    if (state.status.is_at_home !== undefined && state.status.is_at_home !== current_status.is_at_home) return false;
    if (
      state.status.is_carrying_target !== undefined &&
      state.status.is_carrying_target !== current_status.is_carrying_target
    )
      return false;
    if (
      state.status.is_in_capture_range !== undefined &&
      state.status.is_in_capture_range !== current_status.is_in_capture_range
    )
      return false;
    if (state.status.is_in_state !== undefined && state.status.is_in_state !== current_status.current_state)
      return false;
    if (state.status.is_not_in_state !== undefined && state.status.is_not_in_state === current_status.current_state)
      return false;

    return true;
  });
  if (!next_state_or_undefined) return;
  entity.behavior_ai_component.status.current_state = next_state_or_undefined.state;
}
