/**
 * @typedef { import("../../engine.mjs").Game } Game
 * @typedef { import("../../types").Entity } Entity
 */

/**
 * Executes the AI Patrol Behavior for the given entity
 * @param {Game} game
 * @param {Entity} entity
 * @param {number} elapsedTicks
 * @returns {string | undefined}
 */
export function scanAIBehavior(game, entity, elapsedTicks) {
  if (!entity.behavior_ai_component) return;
  if (entity.behavior_ai_component.status.current_state !== 'scan') return;
  if (!entity.behavior_ai_component.configuration.scan_in_place) return;
  if (!entity.controller_movement_component) return;
  if (!entity.position_component) return;
  if (!entity.angle_component) return;

  entity.controller_movement_component.delta_x = 0;
  entity.controller_movement_component.delta_y = 0;

  entity.behavior_ai_component.status.tick_counter += elapsedTicks;
  if (
    entity.behavior_ai_component.status.tick_counter <
    entity.behavior_ai_component.configuration.scan_in_place.ticks_before_increment
  ) {
    return;
  }

  entity.behavior_ai_component.status.tick_counter -=
    entity.behavior_ai_component.configuration.scan_in_place.ticks_before_increment;
  entity.behavior_ai_component.status.increment_counter++;

  const modifier =
    (entity.behavior_ai_component.configuration.scan_in_place.rotation_percent_per_increment * Math.PI) / 50;
  entity.angle_component.angle += modifier;
  if (entity.angle_component.angle < 0) entity.angle_component.angle += 2 * Math.PI;
  entity.angle_component.angle %= 2 * Math.PI;

  if (
    entity.behavior_ai_component.status.increment_counter >
    entity.behavior_ai_component.configuration.scan_in_place.increments_before_event
  ) {
    entity.behavior_ai_component.status.increment_counter = 0;
    return 'scan_cycle_finished';
  }
}
